/*
 * Copyright (C) 2002  Pekka Enberg <Pekka.Enberg@cs.helsinki.fi>
 *
 * Distributed under the terms of the GNU General Public License
 * version 2 or later.
 */
#include <stdio.h>
#include "buffer.h"
#include "indent.h"

#define DEFAULT_INDENT_CHAR ' '
#define DEFAULT_NUM_INDENT_CHARS 4

char indent_char     = DEFAULT_INDENT_CHAR;
int num_indent_chars = DEFAULT_NUM_INDENT_CHARS;
/*
 * Force newline options.
 */
bool force_newline_after_start_tag  = false;
bool force_newline_before_start_tag = false;
bool force_newline_before_end_tag   = false;
bool force_newline_after_end_tag    = false;

static FILE * indent_output;
extern FILE * yyin;
extern void yylex();

#define BUFFER_INITIAL_CAPACITY 1024

/*
 * Uh, oh, we're communicating with yacc trough a global variable
 * called "buffer".
 */
struct buffer __buffer;
struct buffer * buffer = &__buffer;

void indent(FILE * input, FILE * output)
{
    buffer_init(buffer, BUFFER_INITIAL_CAPACITY);

    yyin = input;
    indent_output = output;
    yylex();
    buffer_flush(buffer, indent_output);

    buffer_release(buffer);
}

/* Print indent characters.  */
void print_indent(FILE * output, int indent_level)
{
    int i;

    for (i = 0; i < (num_indent_chars * indent_level); i++) {
	fputc(indent_char, output);
    }
}

void xml_declaration(struct buffer * buffer, const char * text)
{
    buffer_push(buffer, text);
}

void cdata_section(struct buffer * buffer, const char * text)
{
    buffer_push(buffer, text);
}

void doctype_declaration(struct buffer * buffer, const char * text)
{
    buffer_push(buffer, text);
}

void force_newline(struct buffer * buffer)
{
    newline(buffer, "\n");
}

static int indent_level = 0;
static int indent_level_delta = 0;

void start_tag(struct buffer * buffer, const char * text)
{
    if (force_newline_before_start_tag) force_newline(buffer);
    buffer_push(buffer, text);
    indent_level_delta++;
    if (force_newline_after_start_tag) force_newline(buffer);
}

void end_tag(struct buffer * buffer, const char * text)
{
    if (force_newline_before_end_tag) force_newline(buffer);
    buffer_push(buffer, text);
    indent_level_delta--;
    if (force_newline_after_end_tag) force_newline(buffer);
}

void empty_element_tag(struct buffer * buffer, const char * text)
{
    buffer_push(buffer, text);
}

void comment(struct buffer * buffer, const char * text)
{
    buffer_push(buffer, text);
}

void newline(struct buffer * buffer, const char * text)
{
    if (indent_level_delta < 0) {
	indent_level += indent_level_delta;
    }
    print_indent(indent_output, indent_level);
    buffer_push(buffer, text);
    buffer_flush(buffer, indent_output);

    if (indent_level_delta > 0) {
	indent_level += indent_level_delta;
    }
    indent_level_delta = 0;
}

void character_text(struct buffer * buffer, const char * text)
{
    buffer_push(buffer, text);
}

